
/****************************************************************
 *
 *	rtp_h263.c		(C) Copyright Netgem 2008
 *	RTP H263 demuxer
 *
 ****************************************************************/

#include "rtp_h263.h"
#include "libavcodec/bitstream.h"

//#define DEBUG

static int h263_handle_packet(PayloadContext *data,
                              AVStream *st,
                              AVPacket * pkt,
                              uint32_t * timestamp,
                              const uint8_t * buf,
                              int len, int flags)
{
    GetBitContext getbitcontext;
    int picture_start, vrc, plen, pebit;
    uint8_t start_sequence[]= { 0, 0 };
    int err;

    if (len < 2)
        return -1;
    init_get_bits(&getbitcontext, buf, len*8);

    get_bits(&getbitcontext, 5); /* Reserved bits */
    picture_start = get_bits(&getbitcontext, 1);
    vrc = get_bits(&getbitcontext, 1); /* Video Redundancy Coding */
    plen = get_bits(&getbitcontext, 6); /* Length, in bytes, of the extra picture header */
    pebit = get_bits(&getbitcontext, 3); /* number of bits that shall be ignored in the last
                                            byte of the picture header */
    buf+=2;
    len-=2;

    if (vrc) {
        int tid, trun, sync_frame;

        if (len<1)
            return -1;

        tid = get_bits(&getbitcontext, 2);
        trun = get_bits(&getbitcontext, 4);
        sync_frame = get_bits(&getbitcontext, 1);

        buf+=1;
        len-=1;
    }

    dprintf(NULL, "h263_handle_packet p=%d v=%d plen=%d pbit=%d\n",
           picture_start, vrc, plen, pebit);

    if (picture_start) {
        err = av_new_packet(pkt, len + sizeof(start_sequence));
        if (err < 0)
            return -1;
        memcpy(pkt->data, start_sequence, sizeof(start_sequence));
        memcpy(pkt->data + sizeof(start_sequence), buf, len);
    }
    else {
        err = av_new_packet(pkt, len);
        if (err < 0)
            return -1;
        memcpy(pkt->data, buf, len);
    }

    return 0;
}

/**
This is the structure for expanding on the dynamic rtp protocols (makes everything static. yay!)
*/
RTPDynamicProtocolHandler ff_h263_1998_dynamic_handler = {
    "H263-1998",
    CODEC_TYPE_VIDEO,
    CODEC_ID_H263,
    .parse_packet = h263_handle_packet
};
RTPDynamicProtocolHandler ff_h263_2000_dynamic_handler = {
    "H263-2000",
    CODEC_TYPE_VIDEO,
    CODEC_ID_H263,
    .parse_packet = h263_handle_packet
};
