

#ifndef FFMPEG_RTSP_INTERNAL_H
#define FFMPEG_RTSP_INTERNAL_H

#include "network.h"
#include "rtp.h"

/* Netgem:
 * moved out of rtsp.c because rtp_xxx.c needs to know this structure */
typedef struct RTSPStream {
    URLContext *rtp_handle; /* RTP stream handle */
    void *tx_ctx; /* RTP/RDT parse context */

    int stream_index; /* corresponding stream index, if any. -1 if none (MPEG2TS case) */
    int interleaved_min, interleaved_max;  /* interleave ids, if TCP transport */
    char control_url[1024]; /* url for this stream (from SDP) */
    char control_id[64]; /* Netgem */

    int sdp_port; /* port (from SDP content - not used in RTSP) */
    struct in_addr sdp_ip; /* IP address  (from SDP content - not used in RTSP) */
    int sdp_ttl;  /* IP TTL (from SDP content - not used in RTSP) */
    int sdp_payload_type; /* payload type - only used in SDP */
    rtp_payload_data_t rtp_payload_data; /* rtp payload parsing infos from SDP */

    RTPDynamicProtocolHandler *dynamic_handler; ///< Only valid if it's a dynamic protocol. (This is the handler structure)
    PayloadContext *dynamic_protocol_context; ///< Only valid if it's a dynamic protocol. (This is any private data associated with the dynamic protocol)

    /* Netgem: use dynamic rtp packet length */
    int rtp_blocksize; /* max RTP packet length */
    uint8_t *rtp_packet; /* RTP packet */
    /* End Netgem */

} RTSPStream;

#endif /* FFMPEG_RTSP_INTERNAL_H */
