#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include "obj.h"
#include "util.h"
#include "module.h"

/* Conditionally add the symbols from the given symbol set to the
   new module.  */

int add_symbols_from(struct obj_file *f, int idx,
                     struct new_module_symbol *syms, size_t nsyms)
{
  struct new_module_symbol *s;
  size_t i;
  int used = 0;

  for (i = 0, s = syms; i < nsyms; ++i, ++s)
    {
      /* Only add symbols that are already marked external.  If we
	 override locals we may cause problems for argument initialization.
	 We will also create a false dependency on the module.  */

      struct obj_symbol *sym;
      if (strncmp((char *)s->name, "GPLONLY_", 8) == 0)
          s->name = (unsigned long)((char *)s->name + 8);
      sym = obj_find_symbol(f, (char *)s->name);
      if (sym && ! ELFW(ST_BIND)(sym->info) == STB_LOCAL)
	{
	  sym = obj_add_symbol(f, (char *)s->name, -1,
			       ELFW(ST_INFO)(STB_GLOBAL, STT_NOTYPE),
			       idx, s->value, 0);

	  /* Did our symbol just get installed?  If so, mark the
	     module as "used".  */

	  if (sym->secidx == idx)
	    used = 1;
	}
    }

  return used;
}

int new_create_this_module(struct obj_file *f, const char *m_name)
{
  struct obj_section *sec;

  sec = obj_create_alloced_section_first(f, ".this", tgt_sizeof_long,
					 sizeof(struct new_module));
  memset(sec->contents, 0, sizeof(struct new_module));

  obj_add_symbol(f, "__this_module", -1, ELFW(ST_INFO)(STB_LOCAL, STT_OBJECT),
		 sec->idx, 0, sizeof(struct new_module));

  obj_string_patch(f, sec->idx, offsetof(struct new_module, name), m_name);

  return 1;
}

/* add one element to a dynamic array */
void __dynarray_add(unsigned long **tab_ptr, int *nb_ptr, unsigned long elem)
{
    int nb, nb_alloc;
    unsigned long *tab;

    nb = *nb_ptr;
    tab = *tab_ptr;
    if ((nb & (nb - 1)) == 0) {
        if (nb == 0)
            nb_alloc = 1;
        else
            nb_alloc = nb * 2;
        tab = realloc(tab, nb_alloc * sizeof(unsigned long));
        *tab_ptr = tab;
    }
    tab[nb++] = elem;
    *nb_ptr = nb;
}

int get_kernel_symbols(struct new_module_symbol **ksyms_ptr, 
                       int *nksyms_ptr)
{
  struct new_module_symbol *syms, *s;
  size_t nsyms, ret, j, bufsize;
  
  /* Collect the kernel's symbols.  */

  syms = xmalloc(bufsize = 16*1024);
retry_kern_sym_load:
  if (query_module(NULL, QM_SYMBOLS, syms, bufsize, &ret))
    {
      if (errno == ENOSPC)
	{
	  syms = xrealloc(syms, bufsize = ret);
	  goto retry_kern_sym_load;
	}
      error("kernel: QM_SYMBOLS: %m");
      return 0;
    }
  *nksyms_ptr = nsyms = ret;
  *ksyms_ptr = syms;

  for (j = 0, s = syms; j < nsyms; ++j, ++s)
    s->name += (unsigned long)syms;
  return 1;
}


