/*
 * SDPL sdplcalc.c: Utility to calculate the "best" bloc size and 
 *                  number of blocs in a layer.
 *
 *   Copyright (C) 2001, 2002, 2003, 2004, Jocelyn MAYER <l_indien@magic.fr>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * Simple Disk Partition Layer
 *
 * Layer best parameters estimation
 *
 */

/* 
 * Usage: sdplcalc size
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>

#include <linux/types.h>
#include <linux/version.h>

#include <linux/sdpl.h>

void usage(const char *name)
{
    fprintf(stderr, "%s [-e] size\n", name);
    fprintf(stderr, "  -e: embedded layer\n");
    fprintf(stderr, "  size given in kBytes\n");
}

int main (int argc, char **argv)
{
    unsigned long size, reserved;
    int found, min, max, n;
    int size_arg = 1;
    int embedded = 0;
    int sizetype = sizeof(std_type) * 8;

    if (argc == 3) {
        if(strcmp(argv[1], "-e") != 0) {
            usage(argv[0]);
            return -1;
        }
        min = 47;
        max = 64;
        reserved = 1024;
        embedded = 1;
        sizetype = 16;
        size_arg = 2;
    } else 
#if 0
        if (argc != 2) {
        usage(argv[0]);
        return -1;
    }
#else
    {
        fprintf(stderr, "Sorry, only embedded layers for now...\n");
        return -1;
    }
#endif

    size = atol(argv[size_arg]);
    if (size < 2) {
        fprintf(stderr, "Incorrect size: %s\n", argv[size_arg]);
        return -1;
    }

    size *= 1024;
    found = get_best_size(size, &reserved, min, max, &n, sizetype,
                          embedded);
    printf("For size = %lu bytes (%lu) - found %d blocs - %d s/bloc\n\t"
           "%lu bytes reserved - %lu bytes lost (%lu/10000)\n",
           size, size >> 10, found, n, reserved, size - (n * found * 512),
           ((size - (n * found * 512)) * 10000) / size);

    return 0;
}
