/*
 * bd_remap.h
 *
 * This module intend to remap a bloc device into memory.
 * The applecation of this device is to manipulate
 * the root partition while the system is running,
 * with no risk to crash if the kernel or an application
 * intend to access to a file on the device.
 */
/*
 * Copyright (C) 2003-2004 by Jocelyn Mayer <l_indien@magic.fr>
 * distributed under the GNU Public License
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/* History:
 * 2003-01-26: Created
 */
#ifndef __BD_REMAP_H__
#define __BD_REMAP_H__

#ifndef PATH_MAX
#define PATH_MAX 1024
#endif

/* Userland structure */
typedef struct user_remap_t {
    unsigned int dev;
    unsigned int bloc_dec;
    unsigned long nb_blocs;
    int *type;
} user_remap_t;

/* Blocs remap flags */
enum remap_flags {
    REMAP_COPY     = 0,
    REMAP_EMPTYFF  = -1,
    REMAP_EMPTY    = -2,
    REMAP_DISCARD  = -3,
    REMAP_NOREMAP  = -4,
    REMAP_INIT     = -5,
};

/* IOCTLs */
#define REMAP_IOCTL_TYPE 'r'
#define REMAP_DOMAP   _IOWR(REMAP_IOCTL_TYPE, 1, user_remap_t)
#define REMAP_DOUNMAP _IOWR(REMAP_IOCTL_TYPE, 2, user_remap_t)
#define REMAP_GETMAP  _IOWR(REMAP_IOCTL_TYPE, 3, user_remap_t)

#define REMAP_MAJOR 51
#define REMAP_BLOC_SIZE 1024U
#define REMAP_MAX_MINORS 4

#ifdef __KERNEL__

#if defined(USE_BDREMAP_MUTEXES)
struct mutex {
    /*
     * Note that any negative value of count is equivalent to 0,
     * but additionally indicates that some process(es) might be
     * sleeping on `wait'.
     */
    atomic_t count;
    wait_queue_head_t wait;
};

typedef struct mutex mutex_t;
#else
typedef struct semaphore mutex_t;
#endif

/* Kernels internal structures */
typedef struct rem_dev_t {
    mutex_t lock;
    kdev_t device;
    unsigned int bloc_dec;
    unsigned long nb_blocs;
    unsigned long nb_mapped;
    int (*default_request)(request_queue_t *q, int rw,
                           struct buffer_head *bh);
    int *btype;
} rem_dev_t;

#endif /* __KERNEL__ */

#endif /* __BD_REMAP_H__ */
